#!/usr/bin/env python3
'''
Copyright <2018> <Danny Antaki>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'''
__version__='0.0.1'
from pyfreq.vcf import allele_freq
from argparse import RawTextHelpFormatter
from time import time
import argparse,os,shutil,sys
__useage___="""
________   __  ___________ _____ _____ 
| ___ \ \ / /    ___| ___ \  ___|  _  |
| |_/ /\ V /  | |_  | |_/ /  |__| | | |
|  __/  \ /   |  _| |    /|  __|| | | |
| |     | |   | |   | |\ \| |___\ \/' /
\_|     \_/   \_|   \_| \_\____/ \_/\_\
                                     
pyfreq: compute allele frequency for VCF files
Version {}    Author: Danny Antaki <dantaki at ucsd dot edu>    github.com/dantaki/pyfreq

  pyfreq   -v <in.vcf>   -f <in.fam> [-LoPh]

input arguments: 
 
  -v, -vcf       PATH     vcf file  
  -f, -fam       PATH     plink fam file
 
optional arguments:
  
  -L, -log       PATH    log file for standard error messages [default: STDOUT]
  -o, -out       PATH    output prefix [default: pyfreq.out]
  -P, -prog              display a progress bar

  -h, -help              show this message and exit

""".format(__version__)
def main():
	init_time = int(time())
	parser = argparse.ArgumentParser(formatter_class=RawTextHelpFormatter,usage=__useage___,add_help=False)
	in_args, opt_args = parser.add_argument_group('input arguments'),parser.add_argument_group('optional arguments')
	in_args.add_argument('-v','-vcf',type=str,default=None,required=True)
	in_args.add_argument('-f','-fam',type=str,default=None,required=True)
	opt_args.add_argument('-L','-log',default=None,required=False)
	opt_args.add_argument('-o','-out',required=False,default="pyfreq.out",type=str)
	opt_args.add_argument('-P','-prog',required=False,default=False,action="store_true")
	opt_args.add_argument('-h','-help',required=False,action="store_true",default=False)
	args = parser.parse_args()
	vcf,fam = args.v,args.f
	logfh, ofh, prog = args.L,args.o,args.P
	_help = args.h
	if (_help==True or len(sys.argv)==1):
		print(__useage___)
		sys.exit(0)
	if fam==None:
		print('\nFATAL ERROR: fam file required\n')
		sys.exit(1)
	if vcf==None:
		print('\nFATAL ERROR: vcf file required\n')
		sys.exit(1)
	if logfh!=None:
		lfh = open(logfh,'w')
		sys.stderr=lfh
	allele_freq(fam,vcf,ofh,prog)
	# load founders
	# parse VCF
	# calculate allele freq
if __name__=='__main__': main()


#!python
#cython: language_level=3, boundscheck=False, wraparound=False, cdivision=True
import os,sys
from tqdm import tqdm
def allele_freq(fam,vcf,ofh,prog):
	founders = get_founders(fam)
	sample_index = index_header(vcf,founders)
	del founders
	freqs = iterate_vcf(vcf,sample_index,ofh,prog)
def ferr(f):
	if not os.path.isfile(f):
		print('FATAL ERROR {} does not exist'.format(f))
		sys.stderr.write('FATAL ERROR {} does not exist\n'.format(f))
		sys.exit(1)
def get_founders(fam):
	ferr(fam)
	founders={}
	with open(fam,'r') as f:
		for l in f:
			r = l.rstrip().split('\t')
			if r[2] == "0": founders[r[1]]=True
	return founders
cdef index_header(vcf,founders):
	cdef unsigned int ind
	index={}
	vcf_f=None
	if vcf.endswith('.gz'):
		import gzip
		vcf_f = gzip.open(vcf,'r')
	else: vcf_f = open(vcf,'r')
	with vcf_f as f:
		for l in f:
			if vcf.endswith('.gz'):
				l = l.decode("utf-8")
			r = l.rstrip().split('\t')
			if l.startswith('#CHROM'):
				for ind in range(0,len(r)):
					if founders.get(r[ind])!=None:
						index[ind]=True
			if not l.startswith('#'): break
	return index
cdef iterate_vcf(vcf,sample_index,ofh,prog):
	cdef bint terminate=0
	cdef unsigned short a=0
	cdef unsigned int ac=0
	cdef unsigned int ind=0
	#cdef float af=0.
	cdef float tot = len(sample_index)
	vcf_f=None
	out = open(ofh,'w')
	out.write('ID\tALLELE_COUNT\tALLELE_FREQUENCY\n')
	if vcf.endswith('.gz'):
		import gzip
		vcf_f = gzip.open(vcf,'r')
	else: vcf_f = open(vcf,'r')
	with vcf_f as f:
		tq = tqdm(total=os.path.getsize(vcf),unit='B',unit_scale=True,disable= not prog, file=sys.stdout)
		for l in f:
			tq.update(len(l))
			if vcf.endswith('.gz'):
				l = l.decode("utf-8")
			r = l.rstrip().split('\t')
			if not l.startswith('#'):
				########################
				a=0
				ac=0
				for ind in sample_index:
					gt = r[ind].split(':').pop(0).replace("|","/")
					if '.' in gt: continue
					for allele in gt.split('/'):
						a = int(allele)
						if a > 1: terminate=1
						ac+=a
				if terminate==0:
					out.write("{}\t{}\t{}\n".format(r[2],ac,ac/(2*tot)))
				else: sys.stderr.write('WARNING: multiallelic variant found {}\n'.format(r[2]))


from Cython.Build import cythonize
from setuptools import setup,Extension
import numpy
setup(
	name='pyfreq',
	version='0.0.1',
	url='https://github.com/dantaki/pyfreq',
	author='Danny Antaki',
	author_email='dantaki@ucsd.edu',
	packages=['pyfreq'],
	package_dir={'pyfreq': 'pyfreq/'},
	ext_modules=cythonize([
		Extension('pyfreq.vcf',['pyfreq/vcf.pyx']),
		],
		compiler_directives={
			'boundscheck': False,
			'wraparound': False,
			'cdivision': True,
			'language_level': 3,
		}),
	include_dirs=[numpy.get_include()],
	include_package_data=True,
	scripts= ['pyfreq/pyfreq']

)